MODULE trcini
   !!==========================================================================
   !!
   !!                       *** MODULE trcini ***
   !!
   !! Initialization for passive tracer (for restart or not)  
   !!     
   !!=========================================================================
                                                                                                                      
      !! * Modules used
      !! ==============
      USE ioipsl
      USE oce_trc
      USE trc
      USE sms
      USE lib_mpp

     IMPLICIT NONE
     PRIVATE

    !! * Accessibility
    PUBLIC trc_ini

CONTAINS

#if defined key_passivetrc

 SUBROUTINE trc_ini
!!---------------------------------------------------------------------
!!                       ROUTINE trcini
!!                     ******************
!!  PURPOSE :
!!  ---------
!!     initialization for passive tracer
!!     for restart or not
!!
!!   WORKSPACE :
!!   ---------
!!      ji jj jk
!!
!!   History:
!!   --------
!!                 04/00 (O. Aumont, M.A. Foujols) HAMOCC3 and P3ZD
!!                 03/05  O.Aumont and A.El Moussaoui  F90
!!----------------------------------------------------------------------
!!----------------------------------------------------------------------
!! local declarations
!! ==================

      INTEGER :: ji,jj,jk,jn,jl

!!----------------------------------------------------------------------
!! statement functions
!! ===================

#include "passivetrc_substitute.h90"

!!---------------------------------------------------------------------
!!  OPA.9   
!!---------------------------------------------------------------------

#    if defined key_trc_npzd
#    include "trcini.npzd.h90"
#    elif defined key_trc_lobster1
#    include "trcini.lobster1.h90"
#    elif defined key_trc_hamocc3
#    include "trcini.hamocc3.h90"
#    elif defined key_trc_pisces
#    include "trcini.pisces.h90"
#    else

! Specific initialization to do
!!-----------------------------
#    endif

!! 1. initialization of passives tracers field
!! -------------------------------------------
      DO jn=1,jptra
              trn(:,:,:,jn)=0.e0
              tra(:,:,:,jn)=0.e0
      END DO

#if defined key_trc_diaadd
!! initialization of output 2d and 3d arrays

      DO jn=1,jpdia2d
            trc2d(:,:,jn)=0.e0
      END DO
      DO jn=1,jpdia3d
              trc3d(:,:,:,jn)=0.e0
      END DO
#endif

#if defined key_trc_diabio
   !! initialization of biological trends
      DO jn=1,jpdiabio
              trbio(ji,jj,jk,jn)=0.e0
      END DO
#endif

#if defined key_trc_diatrd
   !! initialization of tracer trends
      DO jl=1,jpdiatrc
        DO jn=1,jptra
                trtrd(:,:,:,jn,jl)=0.e0
        END DO
      END DO
#endif      
      
      IF(lwp) THEN
          WRITE(numout,*) ' '
          WRITE(numout,*) ' trcini: generic initialisation done '
          WRITE(numout,*) ' '
      ENDIF

END SUBROUTINE trc_ini

#else

SUBROUTINE trc_ini

   !! no passive tracers
   !!===================
END SUBROUTINE trc_ini

#endif

END MODULE trcini
